#ifndef _KPILOT_UIDIALOG_H
#define _KPILOT_UIDIALOG_H
/* uiDialog.h                           KPilot
**
** Copyright (C) 2001 by Dan Pilone
** Copyright (C) 2004 by Adriaan de Groot
**
** This class defines a way to add an "about widget" to a tab widget.
**
*/

/*
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU Lesser General Public License as published by
** the Free Software Foundation; either version 2.1 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
** GNU Lesser General Public License for more details.
**
** You should have received a copy of the GNU Lesser General Public License
** along with this program in a file called COPYING; if not, write to
** the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
** MA 02111-1307, USA.
*/

/*
** Bug reports and questions can be sent to kde-pim@kde.org
*/

class QTabWidget;
class QPushButton;
class KAboutData;

class KDE_EXPORT UIDialog
{
public:
	/**
	* This is the function that does the work of adding an about
	* page to a tabwidget. It is made public and static so that
	* it can be used elsewhere wherever tabwidgets appear.
	*/
	static  void addAboutPage(QTabWidget *,
		KAboutData *data=0L,
		bool aboutbutton=false);

	/**
	* This creates the actual about widget. Again, public & static so
	* you can slap in an about widget wherever.
	*/
	static QWidget *aboutPage(QWidget *parent, KAboutData *data=0L);
} ;

#endif
