<!DOCTYPE kpartgui>
<kpartgui version="4" name="kmreadermainwin" >
 <MenuBar>
  <Menu noMerge="1" name="file" >
   <text>&amp;File</text>
   <Action name="file_open" />
   <Action name="file_save_as" />
   <Action name="file_print" />
   <Separator/>
   <Action name="file_close" />
  </Menu>
  <Menu noMerge="1" name="view">
   <text>&amp;View</text>
   <Action name="view_headers"/>
   <Action name="view_attachments"/>
   <Separator/>
   <Action name="view_source"/>
   <Separator/>
   <Action name="toggle_fixedfont"/>
   <Action name="encoding" />
  </Menu>
  <Menu noMerge="1" name="message" >
   <text>&amp;Message</text>
   <Action name="new_message" />
   <Action name="post_message" />
   <Separator/>
   <Action name="reply" />
   <Action name="reply_all" />
   <Action name="reply_author" />
   <Action name="reply_list" />
   <Action name="noquotereply" />
   <Menu name="menubar_message_forward">
     <text>&amp;Forward</text>
     <Action name="message_forward_as_attachment" />
     <Action name="message_forward_inline" />
     <Action name="message_forward_redirect" />
   </Menu>
  </Menu>
  <Menu noMerge="1" name="settings">
   <text>&amp;Settings</text>
   <Merge name="StandardToolBarMenuHandler"/>
   <Separator/>
   <Action name="options_configure_keybinding" group="settings_configure"/>
   <Action name="options_configure_toolbars" group="settings_configure" />
   <Action name="kmail_configure_kmail" group="settings_configure"/>
  </Menu>
 </MenuBar>
 <ToolBar noMerge="1" name="mainToolBar" fullWidth="true" >
  <text>Main Toolbar</text>
  <Action name="file_print" />
  <Separator/>
  <Action name="message_reply_menu" />
  <Action name="message_forward"/>
 </ToolBar>
</kpartgui>
