<!DOCTYPE kpartgui>
<kpartgui name="multisynk" version="1">
  <MenuBar>
    <Menu name="file"><text>&amp;File</text>
      <Action name="new"/>
      <Separator/>
      <Action name="sync"/>
      <Merge/>
      <Action name="quit"/>
    </Menu>
    <Menu name="edit"><text>&amp;Edit</text>
      <Action name="edit"/>
      <Action name="delete"/>
    </Menu>
    <Menu name="tools"><text>&amp;Tools</text>
      <Action name="log"/>
    </Menu>
  </MenuBar>

  <ToolBar name="mainToolBar"><text>Main</text>
    <Action name="new"/>
    <Action name="edit"/>
    <Action name="delete"/>
    <Separator/>
    <Action name="log"/>
    <Separator/>
    <Action name="sync"/>
  </ToolBar>
</kpartgui>
