<!DOCTYPE kpartgui>
<kpartgui name="kups" version="1">
<MenuBar>
  <Menu name="action"><text>&amp;Action</text>
    <Action name="view_file" />
    <Action name="view_file_as" />
    <Separator name="separator_0" />
    <Action name="extract_file" />
    <Action name="extract_file_to" />
    <Separator name="separator_1" />
    <Action name="extract_all_files" />
    <Separator name="separator_2" />
    <Action name="properties_file" />
	<Separator />
	<Action name="msg_properties" />
	<Action name="msg_text" />
	<Action name="msg_save" />
  </Menu>
  <Menu name="settings"><text>&amp;Settings</text>
    <Action name="options_default_dir" append="save_merge"/>
  </Menu>
</MenuBar>
<ToolBar name="mainToolBar">
  <text>Main Toolbar</text>
  <Action name="open_file" />
  <Separator name="separator_3" />
  <Action name="view_file" />
  <Action name="properties_file" />
  <Separator name="separator_4" />
  <Action name="extract_file_to" />
  <Action name="extract_all_files" />
  <Separator name="separator_5" />
  <Action name="msg_properties" />
  <Action name="msg_text" />
</ToolBar>
</kpartgui>
