<!DOCTYPE kpartgui>
<kpartgui name="kitchensync" version="5">
<MenuBar>
  <Menu name="file"><text>&amp;File</text>
	<Merge/>
	<Action name="sync"/>
	<Separator/>
	<Action name="backup"/>
 	<Action name="restore"/>
        <Separator/>
	<Action name="quit"/>
  </Menu>
  <Menu name="settings"><text>&amp;Settings</text>
	<Action name="configure"/>
	<Action name="config_profile"/>
	<Action name="config_current"/>
	<Seperator/>
  </Menu>
</MenuBar>
<ToolBar name="mainToolBar"><text>Main</text>
   <Action name="sync"/>
   <Action name="select_kon"/>
   <Action name="select_prof"/>
</ToolBar>
</kpartgui>
